@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -57,6 +57,13 @@
 
 #include <dbus/dbus-glib-lowlevel.h>
 
+#ifdef G_OS_WIN32
+#include <io.h>
+#include <conio.h>
+#define _WIN32_WINNT 0x0500 
+#include <windows.h>
+#endif
+
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
 #define safe_g_hash_table_insert g_hash_table_insert
@@ -112,8 +119,6 @@ static GConfDatabase*       obtain_database (GSList *addresses,
 static void                 drop_old_databases (void);
 static gboolean             no_databases_in_use (void);
 
-static void gconf_handle_segv (int signum);
-
 /*
  * Flag indicating that we are shutting down, so return errors
  * on any attempted operation. We do this instead of unregistering with
@@ -424,22 +429,6 @@ signal_handler (int signo)
   ++in_fatal;
   
   switch (signo) {
-    /* Fast cleanup only */
-  case SIGSEGV:
-#ifdef SIGBUS
-  case SIGBUS:
-#endif
-  case SIGILL:
-    enter_shutdown ();
-#ifndef G_OS_WIN32
-    if (g_getenv ("DISPLAY"))
-      gconf_handle_segv (signo);
-#else
-    gconf_handle_segv (signo);
-#endif
-    abort ();
-    break;
-
   case SIGFPE:
 #ifdef SIGPIPE
   case SIGPIPE:
@@ -712,6 +701,19 @@ get_on_system_bus (void)
 }
 #endif  /* ENABLE_DEFAULTS_SERVICE */
 
+#ifdef G_OS_WIN32
+
+static void
+wait_console_window (void)
+{
+  SetConsoleTitle ("GConf daemon exiting. Type any character to close this window.");
+  printf ("\n"
+	  "(GConf daemon exiting. Type any character to close this window)\n");
+  _getch ();
+}
+
+#endif
+
 int 
 main(int argc, char** argv)
 {
@@ -762,6 +764,45 @@ main(int argc, char** argv)
   else
     {
       gconf_log_debug_messages = TRUE;
+#ifdef G_OS_WIN32
+      if (fileno (stdout) != -1 &&
+	  _get_osfhandle (fileno (stdout)) != -1)
+	{
+	  /* stdout is fine, presumably redirected to a file or pipe */
+	}
+      else
+	{
+	  int allocated_new_console = FALSE;
+
+	  typedef BOOL (* WINAPI AttachConsole_t) (DWORD);
+
+	  AttachConsole_t p_AttachConsole =
+	    (AttachConsole_t) GetProcAddress (GetModuleHandle ("kernel32.dll"), "AttachConsole");
+
+	  if (p_AttachConsole != NULL)
+	    {
+	      if (!AttachConsole (ATTACH_PARENT_PROCESS))
+		{
+		  if (AllocConsole ())
+		    allocated_new_console = TRUE;
+		}
+
+	      freopen ("CONOUT$", "w", stdout);
+	      dup2 (fileno (stdout), 1);
+	      freopen ("CONOUT$", "w", stderr);
+	      dup2 (fileno (stderr), 2);
+
+	      if (allocated_new_console)
+		{
+		  SetConsoleTitle ("GConf daemon debugging output. You can minimize this window, but don't close it.");
+		  printf ("You asked for debugging output by setting the GCONF_DEBUG_OUTPUT\n"
+			  "environment variable, so here it is.\n"
+			  "\n");
+		  atexit (wait_console_window);
+		}
+	    }
+	}
+#endif
     }
   
   umask (022);
@@ -785,28 +826,16 @@ main(int argc, char** argv)
   act.sa_mask    = empty_mask;
   act.sa_flags   = 0;
   sigaction (SIGTERM,  &act, NULL);
-  sigaction (SIGILL,  &act, NULL);
-  sigaction (SIGBUS,  &act, NULL);
-  sigaction (SIGFPE,  &act, NULL);
   sigaction (SIGHUP,  &act, NULL);
-  sigaction (SIGSEGV, &act, NULL);
-  sigaction (SIGABRT, &act, NULL);
   sigaction (SIGUSR1,  &act, NULL);
 
   act.sa_handler = SIG_IGN;
   sigaction (SIGINT, &act, NULL);
 #else
   signal (SIGTERM, signal_handler);
-  signal (SIGILL,  signal_handler);
-#ifdef SIGBUS
-  signal (SIGBUS,  signal_handler);
-#endif
-  signal (SIGFPE,  signal_handler);
 #ifdef SIGHUP
   signal (SIGHUP,  signal_handler);
 #endif
-  signal (SIGSEGV, signal_handler);
-  signal (SIGABRT, signal_handler);
 #ifdef SIGUSR1
   signal (SIGUSR1,  signal_handler);
 #endif
@@ -2204,7 +2233,7 @@ read_line (FILE *f)
         }
 
       len = strlen (buf);
-      if (buf[len - 1] == '\n')
+      if (len > 0 && buf[len - 1] == '\n')
 	buf[--len] = '\0';
 
       if (retval == NULL)
@@ -2651,10 +2680,3 @@ client_count (void)
     return g_hash_table_size (client_table);
 }
 
-
-static void
-gconf_handle_segv (int signum)
-{
-  return;
-}
-
